@@ -1,12 +1,11 @@
 ################################################################################
-### Part of the R package "polyCub".
-### Free software under the terms of the GNU General Public License, version 2,
-### a copy of which is available at http://www.r-project.org/Licenses/.
+### Internal Functions
 ###
-### Copyright (C) 2009-2014 Sebastian Meyer
-### Time-stamp: <[tools.R] 2014-09-27 10:59 (CEST) by SM>
+### Copyright (C) 2009-2015,2017 Sebastian Meyer
 ###
-### Tiny toolbox of internal function
+### This file is part of the R package "polyCub",
+### free software under the terms of the GNU General Public License, version 2,
+### a copy of which is available at https://www.R-project.org/Licenses/.
 ################################################################################
 
 
@@ -43,9 +42,9 @@ dotprod <- function (x,y) sum(x*y)
 ##' @return \code{sqrt(sum(x^2))}
 ##' @keywords math internal
 vecnorm <- function (x) sqrt(sum(x^2))
-    
+
 ##' Checks if Argument is Scalar
-##' 
+##'
 ##' Check if the argument is scalar, i.e. a numeric vector of length 1.
 ##' @param x any object
 ##' @return logical
@@ -62,9 +61,6 @@ isScalar <- function (x) {
 ##' @import methods
 ##' @import sp
 ##' @importFrom graphics polygon
-##' @importFrom spatstat plot.owin
-## CAVE: need to import plot.owin for compatibility with spatstat <1.33-0,
-##       since plot.owin was not registered as an S3-method for plot
 ## NOTE: we don't import graphics::plot since it is already imported via sp
 plot_polyregion <- function (polyregion, lwd=2, add=FALSE)
 {
@@ -73,13 +69,15 @@ plot_polyregion <- function (polyregion, lwd=2, add=FALSE)
         lapply(polyregion, polygon, lwd=lwd)
         invisible()
     } else if (inherits(polyregion, "gpc.poly")) {
-        if (!isClass("gpc.poly")) library("rgeos") # probably redundant
         plot(polyregion, poly.args=list(lwd=lwd), ann=FALSE, add=add)
     } else {
         if (inherits(polyregion, "Polygon"))
             polyregion <- Polygons(list(polyregion), "ID")
         if (inherits(polyregion, "Polygons"))
             polyregion <- SpatialPolygons(list(polyregion))
+        if (inherits(polyregion, "owin"))
+            ## && ! "plot.owin" %in% getNamespaceInfo("spatstat", "S3methods")
+            plot <- spatstat::plot.owin  # spatstat <1.33-0 has no registration
         ## plot call which works for "SpatialPolygons" and "owin"
         plot(polyregion, lwd=lwd, axes=TRUE, main="", add=add)
     }
@@ -87,7 +85,7 @@ plot_polyregion <- function (polyregion, lwd=2, add=FALSE)
 
 
 ##' Constructs Equally-Spaced Grid
-##' 
+##'
 ##' Construct an equally-spaced grid given a range and the number of cut points
 ##' (one more than the number of resulting bins).
 ##' This is nothing else than \code{seq(range[1], range[2], length.out=n)}.
